@@ -4,3 +4,5 @@ from mrtutils.mrtYamlHelper import *
 from mrtutils.device import *
 from mrtutils.gatt_profile import *
 from mrtutils.updates import *
+from mrtutils.mrtDocHelper import *
+from mrtutils.menuconfig import *
