@@ -28,27 +28,23 @@ from threading import Thread
 from urllib.request import urlopen
 import yaml
 
-def getGitFileText(url, file):
-    txt=""
-    try:
-        str = 'git archive --remote=' + url +' HEAD '+ file + ' | tar -xOf - ' + file
-        txt = subprocess.check_output(str,shell=True, encoding='utf8')
-    except subprocess.CalledProcessError as e:
-        print(" Repo at " + url+ " does not contain " + file)
+devObjBlacklist= ['registers', 'fields', 'packages', 'configs', 'variants', 'pins']
 
-    return txt
-
-def getBitbucketFile(account, repo, file):
+def getRemoteFileText(url, file):
     txt=""
-    try:
-        response = urlopen("https://bitbucket.org/" + account + "/" + repo + "/raw/master/" + file)
-        html_content = response.read()
-        encoding = response.headers.get_content_charset('utf-8')
-        txt = html_content.decode(encoding)
-    except:
-        print("bitbucket.org/" + account + "/" + repo + "/raw/master/" + file + "  not found")
-    
-    return txt
+
+    repo= Repo(url, True, False)
+    return repo.getFileText(file)
+
+
+
+def copyRepoFile(url, file, outfile):
+    txt = getRemoteFileText(url,file)
+    dst = open( outfile , "w")
+    dst.write(txt)
+    dst.close()
+    #print("Pulled {0} from {1}".format(file,url))
+
 
 class Submodule:
     def __init__(self, path, url):
@@ -64,6 +60,7 @@ class Submodule:
         self.repo_name = url.split('/')[-1].replace(".git","")
         self.yaml = {}  
         self.repo = None
+        self.submodrepo = Repo(self.url, True, False)
 
 
     def checkFor(self,mods):
@@ -75,13 +72,9 @@ class Submodule:
 
 
     def getReadMe(self):
-        if self.url.find("bitbucket"):
-            self.readme = getBitbucketFile('uprev',self.repo_name, 'README.md')
-        else:
-            self.readme = getGitFileText('ssh://' + self.url.replace(':','/'), 'README.md')
 
-        if(self.readme == ""):
-            self.readme = "# No README.md is available for this module"
+        return self.submodrepo.getReadMe()
+
 
     def loadYaml(self, yamlObj):
 
@@ -91,9 +84,10 @@ class Submodule:
                     self.requirements.append('Common')
                 else:
                     self.requirements.append(req)
+        
+        self.mrtprops = yamlObj
 
-
-    def getYaml(self):
+    def getOtherYaml(self, file):
 
         yamlText = ""
         ret = {}
@@ -110,22 +104,18 @@ class Submodule:
             ret['doc'] = self.path+"/README.md"
 
 
+        try:
+            yamlText = self.submodrepo.getFileText(file)
 
-        if self.repo.isRemote:
-            try:
-                if self.url.find("bitbucket"):
-                    yamlText = getBitbucketFile('uprev',self.repo_name, 'mrt.yml')
-                else:
-                    yamlText = getGitFileText('ssh://' + self.url.replace(':','/'), 'mrt.yml')
+        except:
+            print("couldnt load yaml")
+
+        #Get device file if it exists
+        try:
+            devText = self.submodrepo.getFileText(file)
+        except:
+            print("couldnt load yaml")
 
-            except:
-                print("couldnt load yaml")
-        else:
-            try:
-                f = open(self.path + '/mrt.yml', 'r')
-                yamlText = f.read()
-            except:
-                print("No mrt.yml for " + self.name)
         
         try:
             yamlText = yamlText.replace('\t',' ')
@@ -133,10 +123,61 @@ class Submodule:
         except:
             print("Parsing error: " +self.name )
 
+        
+
         if not yamlObj is None:
             for key, value in yamlObj.items():
                 if not key == 'name':
                     ret[key] = value
+        
+        return ret
+
+
+    def getYaml(self):
+
+        yamlText = ""
+        ret = {}
+        yamlObj = {}
+
+        devText =""
+        devObj = {}
+
+        ret['url'] = self.url
+        ret['path'] = self.path 
+        ret['name'] = self.name
+
+        if os.path.isfile(self.path+"/README.rst"):
+            ret['doc'] = self.path+"/README.md"
+        
+        elif os.path.isfile(self.path+"/README.md"):
+            ret['doc'] = self.path+"/README.md"
+
+
+        yamlText = self.submodrepo.getFileText('mrt.yml')
+        
+        
+        try:
+            yamlText = yamlText.replace('\t',' ')
+            yamlObj = yaml.load(yamlText,  Loader=yaml.FullLoader)
+        except:
+            print("Parsing error: " +self.name )
+
+        
+
+        if not yamlObj is None:
+            for key, value in yamlObj.items():
+                if not key == 'name':
+                    ret[key] = value
+        
+
+        if 'Devices' in self.path:
+            devObj = self.getOtherYaml("device.yml")
+            for key, value in devObj.items():
+                if not key in ret and not key in devObjBlacklist:
+                    ret[key] = value
+                    
+
+        #get other yaml files 
 
         
         return ret
@@ -218,7 +259,7 @@ class RepoDirectory:
         for dir in self.dirs.values():
             dir.printout(lvl+1)
 
-    def copyDocStruct(self, lvl, root ):
+    def copyDocStruct(self, lvl, root , root_index_text = "" ):
 
         if lvl == 0:
             path = root
@@ -226,11 +267,18 @@ class RepoDirectory:
             path = root+"/"+self.name
 
 
-        os.makedirs(path,exist_ok=True)
 
-        indexText = self.name+"\n"
-        for c in self.name:
-            indexText+= "="
+
+        os.makedirs(path,exist_ok=True)
+        indexText =""
+
+        if lvl == 0 and root_index_text != "":
+            indexText = root_index_text 
+        else:
+            indexText = self.name+"\n"
+            for c in self.name:
+                indexText+= "="
+
         indexText+= "\n"
         indexText+= ".. toctree::\n"
         indexText+= "\t:caption: " + self.name+"\n"
@@ -281,9 +329,15 @@ class RepoDirectory:
             
             for req in mod.requirements:
                 fileTxt+="\tselect ENABLE_{0}\n".format(req.upper())
+  
+            # strYaml = yaml.dump(mod.mrtprops)
+            # strYaml = strYaml.replace("\n", "\n\t\t")
+            # fileTxt +="\thelp\n\t\t{0}\n".format(strYaml)
 
             if not mod.getProp("description") == None:
-                fileTxt +="\thelp\n\t\t{0}\n".format(mod.getProp("description"))
+                brief = mod.getProp("description")
+                brief = brief.replace("\n", "\n\t\t")
+                fileTxt +="\thelp\n\t\t{0}\n".format(brief)
 
 
             fileTxt += "\n"
@@ -295,22 +349,44 @@ class RepoDirectory:
 
 
 class Repo:
-    def __init__(self, path, remote = False):
+    def __init__(self, path, remote = False, createdir=True):
+
         self.url = path
         self.path = path
         self.mods =[]
         self.isRemote = remote
-        self.dir = RepoDirectory("root",0)
-        self.isBitbucket = False
         self.rootYaml = {}
-        nodes = self.url.split('/')
-        self.account = nodes[-2]
+        normalUrl = path.replace(':','/')
+        nodes = normalUrl.split('/')
         self.name = nodes[-1].replace(".git","")
+        self.host = ''
+
         if "bitbucket" in path:
-            self.isBitbucket = True
+            self.host = 'bitbucket'
+            self.account =  nodes[-2]
+        if "github" in path:
+            self.host = 'github'
+            self.account =  nodes[-2]
+
+        self.dir = None 
+        if createdir:
+            self.dir = RepoDirectory("root",0)
 
     def setRelativePath(self,path):
         self.relativePath = path
+
+    def getDir(self, path):
+
+        nodes = path.split('/')
+        curDir = self.dir 
+        for n in nodes:
+            if n != '.' and n in curDir.dirs:
+                curDir = curDir.dirs[n]
+
+
+        return curDir
+
+
     
     def crossCheckMods(self,repo):
         for mod in self.mods:
@@ -320,22 +396,9 @@ class Repo:
         
         yamlText = ""
         yamlObj = {}
-        
-        if self.isRemote:
-            try:
-                if self.url.find("bitbucket"):
-                    yamlText = getBitbucketFile('uprev',self.name, 'mrt.yml')
-                else:
-                    yamlText = getGitFileText('ssh://' + self.url.replace(':','/'), 'mrt.yml')
 
-            except:
-                print("couldnt load yaml")
-        else:
-            try:
-                f = open(self.path + '/mrt.yml', 'r')
-                yamlText = f.read()
-            except:
-                print("No mrt.yml for " + self.name)
+        yamlText = self.getFileText('mrt.yml')
+        
         
         try:
             yamlText = yamlText.replace('\t',' ')
@@ -361,11 +424,8 @@ class Repo:
         self.rootYaml = self.getRootYaml()
 
         if self.isRemote:
-            if(self.isBitbucket):
-                data = getBitbucketFile(self.account, self.name, '.gitmodules').replace('\r', '').replace('\n', '').replace('\t', '')
-            else:
-                data = getGitFileText(self.url,'.gitmodules').replace('\r', '').replace('\n', '').replace('\t', '')
-            
+
+            data = self.getFileText('.gitmodules').replace('\r', '').replace('\n', '').replace('\t', '')
             #read in all modules
             regex = re.compile(r'\[(.*?)].*?path = (.*?)url = (.*?\.git)')
             modules = regex.findall(data)
@@ -388,21 +448,50 @@ class Repo:
                 if mod.name in self.rootYaml:
                     mod.loadYaml(self.rootYaml[mod.name])
             self.dir.add(mod, 0)
-        
+    
+    def getFileText(self,file):
+
+        ret = ""
+        if self.isRemote:
+
+            req_url = ""
+
+            if(self.host== 'github'):
+                req_url = "https://raw.githubusercontent.com/" + self.account + "/" + self.name + "/master/" + file
+            elif(self.host == 'bitbucket'):
+                req_url = "https://bitbucket.org/" + self.account + "/" + self.name + "/raw/master/" + file
+  
+            
+            try:
+                response = urlopen(req_url)
+                html_content = response.read()
+                encoding = response.headers.get_content_charset('utf-8')
+                ret = html_content.decode(encoding)
+            except:
+                print(req_url +"  not found")
+        else:
+
+            path = self.path + '/' + file
+            if os.path.isfile(path):
+                try:
+                    file = open(path)
+                    ret = file.read()
+                    file.close()
+                except:
+                    file.close()
+                    print( "Error reading " + path)
+
+        return ret
+
 
     def getReadMe(self):
 
-        data = ""
-        if self.isRemote:
-            if(self.isBitbucket):
-                nodes = self.url.split('/')
-                name = nodes[-1]
-                account = nodes[-2]
-                data = getBitbucketFile(self.account, self.name, 'README.md')
-            else:
-                data = getGitFileText(self.url, 'README.md')
+        ret = self.getFileText('README.md')
 
-        return data
+        if(ret == ""):
+            ret = self.getFileText('README.rst')
+        
+        return ret
 
     def findMod(self, name):
         for mod in self.mods: 
