@@ -17,11 +17,23 @@ extern "C"
 
 /*user-block-top-start*/
 /*user-block-top-end*/
+/* Exported Macros ------------------------------------------------------------*/
+
+/* Exported types ------------------------------------------------------------*/
+
+typedef struct{
+    mrt_gatt_pro_t mPro;
+%for svc in obj.services:
+    ${t.padAfter("{0}_svc_t* m{1};".format(svc.prefix,t.camelCase(svc.name) ), 45)}/* ${svc.desc} */
+%endfor
+}${obj.name.lower()}_profile_t;
 
 /* Exported constants --------------------------------------------------------*/
-/* Exported macro ------------------------------------------------------------*/
+extern ${obj.name.lower()}_profile_t ${obj.name}_profile;
 /* Exported functions ------------------------------------------------------- */
-void ${obj.name.lower()}_profile_init(void);
+mrt_status_t ${obj.name.lower()}_profile_init(void);
+
+mrt_status_t ${obj.name.lower()}_profile_register(void);
 
 /*user-block-functions-start*/
 /*user-block-functions-end*/
